@@ -4,8 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: A Perl Mail Authentication Milter
-our $VERSION = '2.20200319'; # VERSION
-use base 'Net::Server::PreFork';
+our $VERSION = '3.20210113'; # VERSION
 use Mail::Milter::Authentication::Handler;
 use Mail::Milter::Authentication::Metric;
 use Mail::Milter::Authentication::Protocol::Milter;
@@ -18,6 +17,7 @@ use Log::Dispatchouli;
 use Net::DNS::Resolver;
 use Net::IP;
 use Proc::ProcessTable;
+use base 'Mail::Milter::Authentication::Net::ServerPatches';
 use vars qw(@ISA);
 
 
@@ -104,12 +104,12 @@ sub write_to_log_hook($self,$priority,$line,@) {
 
 
 sub idle_loop_hook($self,@) {
-    $self->{'metric'}->master_metric_update( $self );
+    $self->{'metric'}->parent_metric_update( $self );
 }
 
 
 sub pre_loop_hook($self,@) {
-    $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':master';
+    $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':parent';
 
     $self->preload_modules( 'Net::DNS', 'Net::DNS::RR' );
     $self->{'metric'} = Mail::Milter::Authentication::Metric->new($self);
@@ -170,7 +170,8 @@ sub run_n_children_hook($self,@) {
 }
 
 
-sub child_init_hook($self,@) {
+sub child_init_hook {
+    my ( $self,$arg ) = @_;
     srand();
 
     my $config = get_config();
@@ -186,8 +187,15 @@ sub child_init_hook($self,@) {
         }
     }
 
-    $self->loginfo( "Child process $PID starting up" );
-    $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':starting';
+    $arg = '' if !defined $arg;
+    if ( $arg eq 'dequeue' ) {
+        $self->loginfo( "Dequeue process $PID starting up" );
+        $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':dequeue';
+    }
+    else {
+        $self->loginfo( "Child process $PID starting up" );
+        $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':starting';
+    }
 
     my $base;
     if ( $config->{'protocol'} eq 'milter' ) {
@@ -209,7 +217,9 @@ sub child_init_hook($self,@) {
            $protocol &= ~SMFIP_NOBODY;
            $protocol &= ~SMFIP_NOHDRS;
            $protocol &= ~SMFIP_NOEOH;
+           $protocol |= SMFIP_HDR_LEADSPC;
         $self->{'protocol'} = $protocol;
+        $self->{'headers_include_space'} = 0;
 
         my $callback_flags = SMFI_CURR_ACTS|SMFIF_CHGBODY|SMFIF_QUARANTINE|SMFIF_SETSENDER;
         $self->{'callback_flags'} = $callback_flags;
@@ -233,16 +243,25 @@ sub child_init_hook($self,@) {
     $self->setup_handlers();
     $self->{'metric'}->set_versions( $self );
 
+    $self->{'handler'}->{'_Handler'}->metric_count( 'forked_children_total', {}, 1 ) unless $arg eq 'dequeue';
+
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':waiting(0)';
 }
 
 
-sub child_finish_hook($self,@) {
+sub child_finish_hook {
+    my ( $self,$arg ) = @_;
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':exiting';
-    $self->loginfo( "Child process $PID shutting down" );
-    eval {
-        $self->{'handler'}->{'_Handler'}->metric_count( 'reaped_children_total', {}, 1 );
-    };
+    $arg = '' if !defined $arg;
+    if ( $arg eq 'dequeue' ) {
+        $self->loginfo( "Dequeue process $PID shutting down" );
+    }
+    else {
+        $self->loginfo( "Child process $PID shutting down" );
+        eval {
+            $self->{'handler'}->{'_Handler'}->metric_count( 'reaped_children_total', {}, 1 );
+        };
+    }
     $self->destroy_objects();
 }
 
@@ -252,6 +271,15 @@ sub pre_server_close_hook($self,@) {
 }
 
 
+sub dequeue($self,@) {
+    my $config = $self->{ 'config' };
+    my $seconds = $config->{'dequeue_timeout'} // 300;
+    $self->{handler}->{_Handler}->set_overall_timeout( $seconds * 1000000 );
+    $self->{handler}->{_Handler}->top_dequeue_callback();
+    $self->{handler}->{_Handler}->clear_overall_timeout();
+}
+
+
 sub get_client_proto($self,@) {
     my $socket = $self->{server}{client};
     if ($socket->isa("Net::Server::Proto")) {
@@ -511,7 +539,7 @@ sub get_valid_pid($pid_file) {
         }
         if ( $process->pid == $pid ) {
             $found_pid = 1;
-            if ( $process->cmndline eq $Mail::Milter::Authentication::Config::IDENT . ':master' ) {
+            if ( $process->cmndline eq $Mail::Milter::Authentication::Config::IDENT . ':parent' ) {
                 return $pid;
             }
         }
@@ -533,7 +561,10 @@ sub get_valid_pid($pid_file) {
 sub find_process {
     my $process_table = Proc::ProcessTable->new();
     foreach my $process ( $process_table->table->@* ) {
-        if ( $process->cmndline eq $Mail::Milter::Authentication::Config::IDENT . ':master' ) {
+        if ( $process->cmndline eq $Mail::Milter::Authentication::Config::IDENT . ':master' ) { ## Legacy naming, will be removed in later version
+            return $process->pid;
+        }
+        if ( $process->cmndline eq $Mail::Milter::Authentication::Config::IDENT . ':parent' ) {
             return $process->pid;
         }
     }
@@ -612,6 +643,8 @@ sub start($args) {
     my $max_spare_children     = $config->{'max_spare_children'}     || 20;
     my $min_spare_children     = $config->{'min_spare_children'}     || 10;
 
+    setup_config();
+
     my %srvargs;
 
     $srvargs{'no_client_stdout'} = 1;
@@ -650,7 +683,7 @@ sub start($args) {
         my $user  = $config->{'runas'};
         my $group = $config->{'rungroup'};
         if ( $user && $group ) {
-        _warn("run as user=$user group=$group");
+            _warn("run as user=$user group=$group");
             $srvargs{'user'}  = $user;
             $srvargs{'group'} = $group;
         }
@@ -760,7 +793,6 @@ sub start($args) {
                 'ipv'   => '*',
                 'proto' => 'tcp',
             };
-            $srvargs{'child_communication'} = 1;
         }
         elsif ( $type eq 'unix' ) {
             _warn(
@@ -774,7 +806,6 @@ sub start($args) {
                 'port'  => $path,
                 'proto' => 'unix',
             };
-            $srvargs{'child_communication'} = 0;
 
             if ($umask) {
                 umask ( oct( $umask ) );
@@ -799,7 +830,6 @@ sub start($args) {
             'ipv'   => '*',
             'proto' => 'tcp',
         };
-        $srvargs{'child_communication'} = 1;
         _warn( 'Metrics available on ' . $metric_host . ':' . $config->{'metric_port'} );
         _warn( 'metric_host/metric_port are depricated, please use metric_connection/metric_umask instead' );
     }
@@ -808,6 +838,15 @@ sub start($args) {
     $srvargs{'listen'} = $listen_backlog;
     $srvargs{'leave_children_open_on_hup'} = 1;
 
+    if ( $config->{'patch_net_server'} && scalar @ports == 1 ) {
+        my $error = 'Net::Server patches can not be applied when listening on a single port';
+        _warn $error;
+        die;
+    }
+
+    $srvargs{'max_dequeue'} = 1;
+    $srvargs{'check_for_dequeue'} = $config->{'check_for_dequeue'} // 60;
+
     _warn "==========";
     _warn "Starting server";
     _warn "Running with perl $PERL_VERSION";
@@ -867,7 +906,7 @@ sub fatal_global($self,$error) {
     }
     else {
         $self->logerror( "Child process $PID signalling global shut down due to fatal error: $error" );
-        kill 'Term', $ppid;
+        kill 'TERM', $ppid;
     }
     die "$error\n";
 }
@@ -878,8 +917,6 @@ sub setup_handlers($self) {
     my $handler = Mail::Milter::Authentication::Handler->new( $self );
     $self->{'handler'}->{'_Handler'} = $handler;
 
-    $handler->metric_count( 'forked_children_total', {}, 1 );
-
     my $config = $self->{'config'};
     foreach my $name ( $config->{'load_handlers'}->@* ) {
         $self->setup_handler( $name );
@@ -911,7 +948,7 @@ sub setup_handler($self,$name) {
     my $object = $package->new( $self );
     $self->{'handler'}->{$name} = $object;
 
-    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
+    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close dequeue } ) {
         if ( $object->can( $callback . '_callback' ) ) {
             $self->register_callback( $name, $callback );
         }
@@ -939,7 +976,7 @@ sub register_callback($self,$name,$callback) {
 
 
 sub sort_all_callbacks($self) {
-    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
+    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close dequeue } ) {
         $self->sort_callbacks( $callback );
     }
 }
@@ -1120,7 +1157,7 @@ Mail::Milter::Authentication - A Perl Mail Authentication Milter
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 SYNOPSIS
 
@@ -1154,11 +1191,11 @@ Hook which runs to write logs
 
 =head2 I<idle_loop_hook()>
 
-Hook which runs in the master periodically.
+Hook which runs in the parent periodically.
 
 =head2 I<pre_loop_hook()>
 
-Hook which runs in the master before looping.
+Hook which runs in the parent before looping.
 
 =head2 I<run_n_children_hook()>
 
@@ -1176,6 +1213,10 @@ Hook which runs when the child is about to finish.
 
 Hook which runs before the server closes.
 
+=head2 I<dequeue()>
+
+Call the dequeue handlers
+
 =head2 I<get_client_proto()>
 
 Get the protocol of the connecting client.
@@ -1290,7 +1331,7 @@ Given a pid file, check for a valid process ID and return if valid.
 
 =head2 I<find_process()>
 
-Search the process table for an authentication_milter master process
+Search the process table for an authentication_milter parent process
 
 =head2 I<control($command)>
 
