@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Client for connecting back to the authmilter server
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use Mail::Milter::Authentication::Net::Milter;
 use Data::Dumper;
 use Digest::MD5 qw{ md5_base64 };
@@ -202,8 +202,7 @@ sub load_mail {
     my @header_split;
 
     HEADERS:
-    foreach my $dataline ( split ( "\n", $mail_data ) ) {
-        $dataline =~ s/\r?\n$//;
+    foreach my $dataline ( split ( /\r?\n/, $mail_data ) ) {
         # Handle transparency
         if ( $dataline =~ /^\./ ) {
             $dataline = substr( $dataline, 1 );
@@ -351,7 +350,7 @@ Mail::Milter::Authentication::Client - Client for connecting back to the authmil
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 SYNOPSIS
 
