@@ -5,9 +5,9 @@ use strict;
 use warnings;
 ##use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Common configuration handling
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use English;
-use JSON;
+use JSON::XS;
 use TOML;
 use Module::Load;
 use Module::Loaded;
@@ -17,6 +17,7 @@ our @EXPORT_OK = qw{
   get_config
   set_config
   default_config
+  setup_config
 };
 
 
@@ -36,6 +37,7 @@ sub default_config {
         'runas'                           => 'nobody',
         'rungroup'                        => 'nogroup',
         'listen_backlog'                  => 20,
+        'check_for_dequeue'               => 60,
         'min_children'                    => 20,
         'max_children'                    => 200,
         'min_spare_children'              => 10,
@@ -45,6 +47,7 @@ sub default_config {
         'connect_timeout'                 => 30,
         'command_timeout'                 => 30,
         'content_timeout'                 => 300,
+        'dequeue_timeout'                 => 300,
         'addheader_timeout'               => 30,
         'dns_timeout'                     => 10,
         'dns_retry'                       => 2,
@@ -54,7 +57,11 @@ sub default_config {
         'tempfail_on_error_trusted'       => '0',
         'milter_quarantine'               => '0',
         'ip_map'                          => {},
-        'handlers'                        => {}
+        'authserv_id'                     => '',
+        'handlers'                        => {},
+        'cache_dir'                       => '/var/cache/authentication_milter',
+        'spool_dir'                       => '/var/spool/authentication_milter',
+        'lib_dir'                         => '/var/lib/authentication_milter',
     };
 
     require Mail::Milter::Authentication;
@@ -75,6 +82,53 @@ sub default_config {
 }
 
 
+sub setup_config {
+    my $config = get_config();
+
+    my $safe_ident = $IDENT;
+    $safe_ident =~ s/[^a-z0-9]/_/g;
+
+    # Setup some directories
+    foreach my $type ( qw{ cache lib spool } ) {
+        my $dir = $config->{$type.'_dir'};
+        if ( $dir ) {
+            # Value supplied, MUST already be setup
+            # Check that we can use the given directory
+            die $type.'_dir does not exist' if ! -e $dir;
+            die $type.'_dir is not a directory' if ! -d $dir;
+            die $type.'_dir is not a writable' if ! -w $dir;
+        }
+        else {
+            if ( $EUID == 0 ) {
+                # We are root, create in global space
+                $dir = '/var/'.$type.'/authentication_milter';
+                mkdir $dir if ! -e $dir;
+                # Create the subdir for this IDENT
+                $dir .= '/'.$safe_ident;
+                mkdir $dir if ! -e $dir;
+                # Chown if relevant
+                my $user  = $config->{'runas'};
+                if ($user) {
+                    my ($login,$pass,$uid,$gid) = getpwnam($user);
+                    chown $uid, $gid, $dir;
+                }
+            }
+            else {
+                # We are a user! Create something in a temporary space
+                $dir = join( '_',
+                  '/tmp/authentication_milter',
+                  $type,
+                  $EUID,
+                  $safe_ident,
+                );
+                mkdir $dir if ! -e $dir;
+            }
+        }
+        $config->{$type.'_dir'} = $dir;
+    }
+}
+
+
 sub set_config {
     my ( $config ) = @_;
 
@@ -111,7 +165,7 @@ sub load_file {
           || die "Error parsing config file $file";
     }
     else {
-        my $json = JSON->new();
+        my $json = JSON::XS->new();
         $json->relaxed(1);
         $data = $json->decode($text)
           || die "Error parsing config file $file";
@@ -210,7 +264,7 @@ Mail::Milter::Authentication::Config - Load config files for Authentication Milt
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 SYNOPSIS
 
@@ -233,6 +287,10 @@ Return a default configuration including defaults from handler modules.
 This is not the default config used by the system if no config is present, rather it is the config
 which is presented to the user as an example default config when using the help feature.
 
+=head2 I<setup_config()>
+
+Called during startup, setup some config options.
+
 =head2 I<set_config( $config )>
 
 Set the config hashref, primarily used for testing.
