@@ -5,7 +5,7 @@ use strict;
 use warnings;
 ##use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Common constants
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use base 'Exporter';
 
 
@@ -69,6 +69,7 @@ use constant SMFIP_NOBODY       => 0x10;
 use constant SMFIP_NOHDRS       => 0x20;
 use constant SMFIP_NOEOH        => 0x40;
 use constant SMFIP_NONE         => 0x7F;
+use constant SMFIP_HDR_LEADSPC  => 0x100000;
 
 use constant SMFIS_CONTINUE     => 100;
 use constant SMFIS_REJECT       => 101;
@@ -140,6 +141,7 @@ our @EXPORT = qw(
     SMFIP_NOHDRS
     SMFIP_NOEOH
     SMFIP_NONE
+    SMFIP_HDR_LEADSPC
     SMFIS_CONTINUE
     SMFIS_REJECT
     SMFIS_DISCARD
@@ -174,7 +176,7 @@ Mail::Milter::Authentication::Constants - Define and export useful constants
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 SYNOPSIS
 
