@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler superclass
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use Mail::Milter::Authentication::Exception;
 use Mail::Milter::Authentication::Resolver;
 use Date::Format qw{ time2str };
@@ -13,6 +13,7 @@ use MIME::Base64;
 use Mail::SPF;
 use Net::DNS::Resolver;
 use Net::IP;
+use Sereal qw{encode_sereal decode_sereal};
 use Sys::Hostname;
 use Time::HiRes qw{ ualarm gettimeofday };
 
@@ -165,6 +166,41 @@ sub register_metrics {
 }
 
 
+sub top_dequeue_callback {
+    my ( $self ) = @_;
+
+    $self->status('dequeue');
+    $self->set_symbol('C','i','DEQUEUE.'.substr( uc md5_hex( "Authentication Milter Client $PID " . time() . rand(100) ) , -11 ));
+    $self->dbgout( 'CALLBACK', 'Dequeue', LOG_DEBUG );
+    my $config = $self->config();
+    eval {
+        local $SIG{'ALRM'} = sub{ die Mail::Milter::Authentication::Exception->new({ 'Type' => 'Timeout', 'Text' => 'Dequeue callback timeout' }) };
+        if ( my $timeout = $self->get_type_timeout( 'dequeue' ) ) {
+            $self->set_alarm( $timeout );
+        }
+        my $callbacks = $self->get_callbacks( 'dequeue' );
+        foreach my $handler ( @$callbacks ) {
+            $self->dbgout( 'CALLBACK', 'Dequeue ' . $handler, LOG_DEBUG );
+            my $start_time = $self->get_microseconds();
+            $self->get_handler($handler)->dequeue_callback();
+            $self->dbgoutwrite();
+            $self->metric_count( 'time_microseconds_total', { 'callback' => 'dequeue', 'handler' => $handler }, $self->get_microseconds_since( $start_time ) );
+        }
+        $self->set_alarm(0);
+    };
+    if ( my $error = $@ ) {
+        if ( my $type = $self->is_exception_type( $error ) ) {
+            $self->metric_count( 'callback_error_total', { 'stage' => 'dequeue', 'type' => $type } );
+        }
+        else {
+            $self->metric_count( 'callback_error_total', { 'stage' => 'dequeue' } );
+        }
+    }
+    $self->dbgoutwrite();
+    $self->status('postdequeue');
+}
+
+
 sub top_setup_callback {
 
     my ( $self ) = @_;
@@ -235,11 +271,11 @@ sub set_handler_alarm {
     my $remaining = $self->get_time_remaining();
     if ( $remaining < $microseconds ) {
         # This should already be set of course, but for clarity...
-        $self->dbgout( 'Handler tmeout set (remaining used)', $remaining, LOG_DEBUG );
+        $self->dbgout( 'Handler timeout set (remaining used)', $remaining, LOG_DEBUG );
         ualarm( $remaining );
     }
     else {
-        $self->dbgout( 'Handler tmeout set', $microseconds, LOG_DEBUG );
+        $self->dbgout( 'Handler timeout set', $microseconds, LOG_DEBUG );
         ualarm( $microseconds );
     }
 }
@@ -340,29 +376,34 @@ sub _remap_ip_and_helo {
         my $helo_host = $self->{'raw_helo_name'};
         foreach my $ip_map ( sort keys %{ $config->{ 'ip_map' } } ) {
             my $map_obj = Net::IP->new( $ip_map );
-            my $is_overlap = $ip_object->overlaps($map_obj) || 0;
-            if (
-                   $is_overlap == $IP_A_IN_B_OVERLAP
-                || $is_overlap == $IP_B_IN_A_OVERLAP     # Should never happen
-                || $is_overlap == $IP_PARTIAL_OVERLAP    # Should never happen
-                || $is_overlap == $IP_IDENTICAL
-              )
-            {
-                my $mapped_to = $config->{ 'ip_map' }->{ $ip_map };
-                if ( $helo_host && exists $mapped_to->{helo_map} && exists $mapped_to->{helo_map}->{ $helo_host } ) {
-                    # We have a specific HELO mapping for this!
-                    $mapped_to = $mapped_to->{helo_map}->{ $helo_host };
-                    return {
-                        ip => Net::IP->new( $mapped_to->{ip} ),
-                        helo => $mapped_to->{helo},
-                    };
-                }
-                else {
-                    # Remap based on IP Only
-                    return {
-                        ip => Net::IP->new( $mapped_to->{ip} ),
-                        helo => $mapped_to->{helo},
-                    };
+            if ( !$map_obj ) {
+                $self->log_error( 'Core: Could not parse IP '.$ip_map );
+            }
+            else {
+                my $is_overlap = $ip_object->overlaps($map_obj) || 0;
+                if (
+                       $is_overlap == $IP_A_IN_B_OVERLAP
+                    || $is_overlap == $IP_B_IN_A_OVERLAP     # Should never happen
+                    || $is_overlap == $IP_PARTIAL_OVERLAP    # Should never happen
+                    || $is_overlap == $IP_IDENTICAL
+                  )
+                {
+                    my $mapped_to = $config->{ 'ip_map' }->{ $ip_map };
+                    if ( $helo_host && exists $mapped_to->{helo_map} && exists $mapped_to->{helo_map}->{ $helo_host } ) {
+                        # We have a specific HELO mapping for this!
+                        $mapped_to = $mapped_to->{helo_map}->{ $helo_host };
+                        return {
+                            ip => Net::IP->new( $mapped_to->{ip} ),
+                            helo => $mapped_to->{helo},
+                        };
+                    }
+                    else {
+                        # Remap based on IP Only
+                        return {
+                            ip => Net::IP->new( $mapped_to->{ip} ),
+                            helo => $mapped_to->{helo},
+                        };
+                    }
                 }
             }
         }
@@ -375,8 +416,13 @@ sub remap_connect_callback {
     $self->{'raw_ip_object'} = $ip;
     my $ip_remap = $self->_remap_ip_and_helo();
     if ( $ip_remap ) {
-        $ip = $ip_remap->{ip};
-        $self->dbgout( 'RemappedConnect', $self->{'raw_ip_object'}->ip() . ' > ' . $ip->ip(), LOG_DEBUG );
+        if ( !$ip_remap->{ip} ) {
+            $self->log_error( 'Core: Ignored bad IP in remapping' );
+        }
+        else {
+            $ip = $ip_remap->{ip};
+            $self->dbgout( 'RemappedConnect', $self->{'raw_ip_object'}->ip() . ' > ' . $ip->ip(), LOG_DEBUG );
+       }
     }
     $self->{'ip_object'} = $ip;
 }
@@ -530,7 +576,7 @@ sub top_envfrom_callback {
 
     # On MAILFROM
     #...
-    my ( $self, $env_from ) = @_;
+    my ( $self, $env_from, @params ) = @_;
     $self->status('envfrom');
     $self->dbgout( 'CALLBACK', 'EnvFrom', LOG_DEBUG );
     $self->set_return( $self->smfis_continue() );
@@ -552,7 +598,7 @@ sub top_envfrom_callback {
         foreach my $handler ( @$callbacks ) {
             $self->dbgout( 'CALLBACK', 'EnvFrom ' . $handler, LOG_DEBUG );
             my $start_time = $self->get_microseconds();
-            eval { $self->get_handler($handler)->envfrom_callback($env_from); };
+            eval { $self->get_handler($handler)->envfrom_callback($env_from, @params); };
             if ( my $error = $@ ) {
                 $self->handle_exception( $error );
                 $self->exit_on_close( 'Env From callback error ' . $error );
@@ -584,7 +630,7 @@ sub top_envrcpt_callback {
 
     # On RCPTTO
     #...
-    my ( $self, $env_to ) = @_;
+    my ( $self, $env_to, @params ) = @_;
     $self->status('envrcpt');
     $self->dbgout( 'CALLBACK', 'EnvRcpt', LOG_DEBUG );
     $self->set_return( $self->smfis_continue() );
@@ -600,7 +646,7 @@ sub top_envrcpt_callback {
         foreach my $handler ( @$callbacks ) {
             $self->dbgout( 'CALLBACK', 'EnvRcpt ' . $handler, LOG_DEBUG );
             my $start_time = $self->get_microseconds();
-            eval{ $self->get_handler($handler)->envrcpt_callback($env_to); };
+            eval{ $self->get_handler($handler)->envrcpt_callback($env_to, @params); };
             if ( my $error = $@ ) {
                 $self->handle_exception( $error );
                 $self->exit_on_close( 'Env Rcpt callback error ' . $error );
@@ -1173,23 +1219,20 @@ sub get_object {
 
         elsif ( $name eq 'resolver' ) {
             $self->dbgout( 'Object created', $name, LOG_DEBUG );
-            my $config = $self->config();
-            my $timeout           = $config->{'dns_timeout'}           || 8;
-            my $dns_retry         = $config->{'dns_retry'}             || 2;
-            my $resolvers         = $config->{'dns_resolvers'}         || [];
             if ( defined $TestResolver ) {
                 $object = $TestResolver;
                 warn "Using FAKE TEST DNS Resolver - I Hope this isn't production!";
                 # If it is you better know what you're doing!
             }
             else {
-                $object = Mail::Milter::Authentication::Resolver->new(
-                    '_handler'          => $self,
-                    'udp_timeout'       => $timeout,
-                    'tcp_timeout'       => $timeout,
-                    'retry'             => $dns_retry,
-                    'nameservers'       => $resolvers,
-                );
+                my $config = $self->config();
+                my %args;
+                $args{_handler}    = $self;
+                $args{udp_timeout} = $config->{'dns_timeout'}   || 8;
+                $args{tcp_timeout} = $config->{'dns_timeout'}   || 8;
+                $args{retry}       = $config->{'dns_retry'}     || 2;
+                $args{nameservers} = $config->{'dns_resolvers'} if $config->{'dns_resolvers'} && $config->{'dns_resolvers'}->@*;
+                $object = Mail::Milter::Authentication::Resolver->new(%args);
                 $object->udppacketsize(1240);
                 $object->persistent_udp(1);
             }
@@ -1379,6 +1422,68 @@ sub tempfail_on_error {
 
 # Common calls into other Handlers
 
+sub _dequeue_dir($self) {
+    my $config = $self->config();
+    my $dir = $config->{spool_dir}.'/dequeue';
+    mkdir $dir if ! -d $dir;
+    return $dir;
+}
+
+
+{
+    my $queue_index = 1;
+    sub add_dequeue($self,$key,$data) {
+        my $dir = $self->_dequeue_dir;
+        my $fullpath;
+        my $timestamp = join( '.',gettimeofday);
+        my $filename = join( '.',$key,$PID,$timestamp,$queue_index++,'dequeue');
+        $fullpath = "$dir/$filename";
+        my $serialised_data = encode_sereal($data);
+        write_file($fullpath,{atomic=>1},$serialised_data);
+    }
+}
+
+
+sub get_dequeue_list($self,$key) {
+    my @dequeue_list;
+    my $dir = $self->_dequeue_dir;
+    opendir(my $dh, $dir) || die "Failed to open dequeue directory: $!";
+    while (my $file = readdir $dh) {
+        push @dequeue_list, $file if $file =~ /^$key\..*\.dequeue$/;
+    }
+    closedir $dh;
+    return \@dequeue_list;
+}
+
+
+sub get_dequeue($self,$id) {
+    my $dir = $self->_dequeue_dir;
+    my $filepath = join('/',$dir,$id);
+    return if ! -e $filepath;
+    return if ! -f $filepath;
+    my $serialized = scalar read_file($filepath);
+    my $data = decode_sereal($serialized);
+    return $data;
+}
+
+
+sub delete_dequeue($self,$id) {
+    my $dir = $self->_dequeue_dir;
+    my $filepath = join('/',$dir,$id);
+    return if ! -e $filepath;
+    return if ! -f $filepath;
+    unlink $filepath;
+}
+
+
+sub error_dequeue($self,$id) {
+    my $dir = $self->_dequeue_dir;
+    my $filepath = join('/',$dir,$id);
+    return if ! -e $filepath;
+    return if ! -f $filepath;
+    rename $filepath, $filepath . '.err';
+}
+
 
 sub is_local_ip_address {
     my ($self) = @_;
@@ -1676,6 +1781,16 @@ sub get_my_hostname {
 }
 
 
+sub get_my_authserv_id {
+    my ($self) = @_;
+    my $config = $self->config();
+    if ( exists( $config->{'authserv_id'} ) && $config->{'authserv_id'} ) {
+	return $config->{'authserv_id'};
+    }
+    return $self->get_my_hostname();
+}
+
+
 
 # Logging
 
@@ -1837,7 +1952,7 @@ sub add_headers {
 
     my $config = $self->config();
 
-    my $header = $self->get_my_hostname();
+    my $header = $self->get_my_authserv_id();
     my $top_handler = $self->get_top_handler();
 
     my @auth_headers;
@@ -1868,7 +1983,7 @@ sub add_headers {
             $header .= join( ";\n    ", map { $self->_stringify_header( $_ ) } @auth_headers );
         }
         else {
-            $header_obj->set_value( Mail::AuthenticationResults::Header::AuthServID->new()->safe_set_value( $self->get_my_hostname() ) );
+            $header_obj->set_value( Mail::AuthenticationResults::Header::AuthServID->new()->safe_set_value( $self->get_my_authserv_id() ) );
             $header_obj->set_eol( "\n" );
             if ( exists( $config->{'header_indent_style'} ) ) {
                 $header_obj->set_indent_style( $config->{'header_indent_style'} );
@@ -1888,13 +2003,15 @@ sub add_headers {
             $header = $header_obj->as_string();
         }
 
+        $self->prepend_header( 'Authentication-Results', $header );
     }
-    else {
+    elsif ( !$config->{'hide_none'} ) {
         $header .= '; none';
+        $self->prepend_header( 'Authentication-Results', $header );
+    } else {
+        # the result is none and hide_none is set, so we do not add an AR header
     }
 
-    $self->prepend_header( 'Authentication-Results', $header );
-
     if ( my $reason = $self->get_quarantine_mail() ) {
         $self->prepend_header( 'X-Disposition-Quarantine', $reason );
     }
@@ -2047,7 +2164,7 @@ Mail::Milter::Authentication::Handler - Handler superclass
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 DESCRIPTION
 
@@ -2336,6 +2453,10 @@ Manually check the current timeout, and throw if it has passed.
 
 =head1 CALLBACK METHODS
 
+=head2 I<top_dequeue_callback()>
+
+Top level handler for dequeue.
+
 =head2 I<top_setup_callback()>
 
 Top level handler for handler setup.
@@ -2404,6 +2525,32 @@ Called after the Authentication-Results header has been added, but before any ot
 
 =head1 HELPER METHODS
 
+=head2 I<add_dequeue($key,$data)>
+
+Write serialized $data into the queue for later dequeueing
+
+=head2 I<get_dequeue_list($key)>
+
+Return an ArrayRef of all queued items for $key
+
+This may be a list of filenames, or may be a list of some
+other ID, it should not be assumed that this value is
+useful outside of the dequeue methods.
+
+Used in get_dequeue_object and delete_dequeue_object
+
+=head2 I<get_dequeue($id)>
+
+Return a previously queued item
+
+=head2 I<delete_dequeue($id)>
+
+Delete a previously queued item
+
+=head2 I<error_dequeue($id)>
+
+Mark a previously queued item as errored
+
 =head2 I<is_local_ip_address()>
 
 Is the current connection from a local ip address?
@@ -2480,6 +2627,10 @@ Extract all email address from a string as an arrayref.
 
 Return the effective hostname of the MTA.
 
+=head2 I<get_my_authserv_id()>
+
+Return the effective authserv-id. Defaults to hostname if not explicitly set.
+
 =head1 LOGGING METHODS
 
 =head2 I<dbgout( $key, $value, $priority )>
