@@ -4,12 +4,12 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for DKIM
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
-use Mail::DKIM 0.39;
+use Mail::DKIM 1.20200824;
 use Mail::DKIM::DNS;
 use Mail::DKIM::KeyValueList;
-use Mail::DKIM::Verifier 0.39;
+use Mail::DKIM::Verifier;
 
 sub default_config {
     return {
@@ -113,11 +113,8 @@ sub eoh_callback {
                 $UseStrict = 0;
             }
             $dkim = Mail::DKIM::Verifier->new( 'Strict' => $UseStrict );
-            # The following requires Mail::DKIM > 0.4
-            if ( $Mail::DKIM::VERSION >= 0.4 ) {
-                my $resolver = $self->get_object('resolver');
-                Mail::DKIM::DNS::resolver($resolver);
-            }
+            my $resolver = $self->get_object('resolver');
+            Mail::DKIM::DNS::resolver($resolver);
             $self->set_object('dkim', $dkim, 1);
         };
         if ( my $error = $@ ) {
@@ -434,7 +431,7 @@ Mail::Milter::Authentication::Handler::DKIM - Handler class for DKIM
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 DESCRIPTION
 
