@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for DMARC
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 use List::MoreUtils qw{ uniq };
 use Mail::DMARC::PurePerl 1.20160612;
@@ -20,12 +20,17 @@ sub default_config {
         'no_list_reject' => 1,
         'arc_before_list' => 0,
         'whitelisted'    => [],
+        'policy_rbl_lookup' => {},
         'detect_list_id' => 1,
         'report_skip_to' => [ 'my_report_from_address@example.com' ],
         'no_report'      => 0,
+        'hide_report_to' => 0,
         'config_file'    => '/etc/mail-dmarc.ini',
         'no_reject_disposition' => 'quarantine',
         'no_list_reject_disposition' => 'none',
+        'reject_on_multifrom' => 30,
+        'quarantine_on_multifrom' => 20,
+        'skip_on_multifrom' => 10,
     };
 }
 
@@ -107,16 +112,21 @@ sub is_whitelisted {
         }
         else {
             my $whitelisted_obj = Net::IP->new($entry);
-            my $is_overlap = $ip_obj->overlaps($whitelisted_obj) || 0;
-            if (
-                   $is_overlap == $IP_A_IN_B_OVERLAP
-                || $is_overlap == $IP_B_IN_A_OVERLAP     # Should never happen
-                || $is_overlap == $IP_PARTIAL_OVERLAP    # Should never happen
-                || $is_overlap == $IP_IDENTICAL
-              )
-            {
-                $self->dbgout( 'DMARCReject', "Whitelist hit " . $entry, LOG_INFO );
-                $whitelisted = 1;
+            if ( !$whitelisted_obj ) {
+                $self->log_error( 'DMARC: Could not parse whitelist IP '.$entry );
+            }
+            else {
+                my $is_overlap = $ip_obj->overlaps($whitelisted_obj) || 0;
+                if (
+                       $is_overlap == $IP_A_IN_B_OVERLAP
+                    || $is_overlap == $IP_B_IN_A_OVERLAP     # Should never happen
+                    || $is_overlap == $IP_PARTIAL_OVERLAP    # Should never happen
+                    || $is_overlap == $IP_IDENTICAL
+                  )
+                {
+                    $self->dbgout( 'DMARCReject', "Whitelist hit " . $entry, LOG_INFO );
+                    $whitelisted = 1;
+                }
             }
         }
         return $whitelisted if $whitelisted;
@@ -210,11 +220,13 @@ sub _process_arc_dmarc_for {
     }
 
     # Add the Envelope To
-    eval {
-        $dmarc->envelope_to( lc $self->get_domain_from( $self->{'env_to'} ) );
-    };
-    if ( my $error = $@ ) {
-        $self->handle_exception( $error );
+    unless ( $config->{'hide_report_to'} ) {
+        eval {
+            $dmarc->envelope_to( lc $self->get_domain_from( $self->{'env_to'} ) );
+        };
+        if ( my $error = $@ ) {
+            $self->handle_exception( $error );
+        }
     }
 
     # Add the From Header
@@ -294,6 +306,43 @@ sub _process_dmarc_for {
 
     my $config = $self->handler_config();
 
+    if ( exists $self->{'processed'}->{ "$env_domain_from $header_domain" } ) {
+        $self->log_error( "DMARC already processed for $env_domain_from $header_domain" );
+        return;
+    }
+    $self->{'processed'}->{ "$env_domain_from $header_domain" } = 1;
+
+    if ( $config->{'reject_on_multifrom'} ) {
+        if ( scalar keys $self->{'processed'}->%* == $config->{'reject_on_multifrom'} ) {
+            $self->log_error( 'DMARC limit reached, rejecting' );
+            $self->reject_mail( '550 5.7.0 DMARC policy violation' );
+            $self->log_error( "DMARC limit reached, skipping processing for $env_domain_from $header_domain" );
+            return;
+        }
+        elsif ( scalar keys $self->{'processed'}->%* > $config->{'reject_on_multifrom'} ) {
+            $self->log_error( "DMARC limit reached, skipping processing for $env_domain_from $header_domain" );
+            return;
+        }
+    }
+    if ( $config->{'quarantine_on_multifrom'} ) {
+        if ( scalar keys $self->{'processed'}->%* == $config->{'quarantine_on_multifrom'} ) {
+            $self->log_error( 'DMARC limit reached, quarantining' );
+            $self->quarantine_mail( 'Quarantined due to DMARC policy' );
+            $self->log_error( "DMARC limit reached, skipping processing for $env_domain_from $header_domain" );
+            return;
+        }
+        elsif ( scalar keys $self->{'processed'}->%* > $config->{'quarantine_on_multifrom'} ) {
+            $self->log_error( "DMARC limit reached, skipping processing for $env_domain_from $header_domain" );
+            return;
+        }
+    }
+    if ( $config->{'skip_on_multifrom'} ) {
+        if ( scalar keys $self->{'processed'}->%* >= $config->{'skip_on_multifrom'} ) {
+            $self->log_error( "DMARC limit reached, skipping processing for $env_domain_from $header_domain" );
+            return;
+        }
+    }
+
     # Get a fresh DMARC object each time.
     $self->destroy_object('dmarc');
     my $dmarc = $self->get_dmarc_object();
@@ -327,12 +376,14 @@ sub _process_dmarc_for {
     }
 
     # Add the Envelope To
-    eval {
-        $dmarc->envelope_to( lc $self->get_domain_from( $self->{'env_to'} ) );
-    };
-    if ( my $error = $@ ) {
-        $self->handle_exception( $error );
-        $self->log_error( 'DMARC Rcpt To Error ' . $error );
+    unless ( $config->{'hide_report_to'} ) {
+        eval {
+            $dmarc->envelope_to( lc $self->get_domain_from( $self->{'env_to'} ) );
+        };
+        if ( my $error = $@ ) {
+            $self->handle_exception( $error );
+            $self->log_error( 'DMARC Rcpt To Error ' . $error );
+        }
     }
 
     # Add the From Header
@@ -523,50 +574,64 @@ sub _process_dmarc_for {
     }
 
     # Add the AR Header
-    my @comments;
-    if ( !( $config->{'hide_none'} && $dmarc_code eq 'none' ) ) {
-        my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dmarc' )->safe_set_value( $dmarc_code );
+    my $header = Mail::AuthenticationResults::Header::Entry->new()->set_key( 'dmarc' )->safe_set_value( $dmarc_code );
 
-        # What comments can we add?
-        if ( $dmarc_policy ) {
-            push @comments, $self->format_header_entry( 'p', $dmarc_policy );
-            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.published-domain-policy' )->safe_set_value( $dmarc_policy ) );
+    # Do any RBL lookups
+    if ( $config->{'policy_rbl_lookup'} && ( $dmarc_code eq 'pass' || $arc_aware_result eq 'pass' )) {
+        foreach my $rbl ( sort keys $config->{'policy_rbl_lookup'}->%* ) {
+            my $rbl_data = $config->{'policy_rbl_lookup'}->{$rbl};
+            my $rbl_domain = $rbl_data->{'rbl'};
+            my $rbl_result = $self->rbl_check_domain( $header_domain, $rbl_domain );
+            if ( $rbl_result ) {
+                my $txt_result
+                    = exists( $rbl_data->{'results'}->{$rbl_result} ) ? $rbl_data->{'results'}->{$rbl_result}
+                    : exists( $rbl_data->{'results'}->{'*'} )         ? $rbl_data->{'results'}->{'*'}
+                    : 'pass';
+                $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.'.$rbl )->safe_set_value( $txt_result ) );
+            }
         }
-        if ( $dmarc_sub_policy ne 'default' ) {
-            push @comments, $self->format_header_entry( 'sp', $dmarc_sub_policy );
-            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.published-subdomain-policy' )->safe_set_value( $dmarc_sub_policy ) );
-        }
-        if ( $config->{'detect_list_id'} && $self->{'is_list'} ) {
-            push @comments, 'has-list-id=yes';
-        }
-        if ( $dmarc_disposition ) {
-            push @comments, $self->format_header_entry( 'd', $dmarc_disposition );
-            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.applied-disposition' )->safe_set_value( $dmarc_disposition ) );
-        }
-        if ( $dmarc_disposition_evaluated ) {
-            push @comments, $self->format_header_entry( 'd.eval', $dmarc_disposition_evaluated );
-            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.evaluated-disposition' )->safe_set_value( $dmarc_disposition_evaluated ) );
-        }
-        if ( $policy_override ) {
-            push @comments, $self->format_header_entry( 'override', $policy_override );
-            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.override-reason' )->safe_set_value( $policy_override ) );
-        }
-        if ( $arc_aware_result ) {
-            push @comments, $self->format_header_entry( 'arc_aware_result', $arc_aware_result );
-            $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.arc-aware-result' )->safe_set_value( $arc_aware_result ) );
-        }
-
-        if ( @comments ) {
-            $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( join( ',', @comments ) ) );
-        }
-
-        my $policy_used = ( $is_subdomain && $dmarc_sub_policy ne 'default' ) ? 'sp' : 'p';
-        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.policy-from' )->safe_set_value( $policy_used ) );
-
-        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.from' )->safe_set_value( $header_domain ) );
-        $self->_add_dmarc_header( $header );
     }
 
+    # What comments can we add?
+    my @comments;
+    if ( $dmarc_policy ) {
+        push @comments, $self->format_header_entry( 'p', $dmarc_policy );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.published-domain-policy' )->safe_set_value( $dmarc_policy ) );
+    }
+    if ( $dmarc_sub_policy ne 'default' ) {
+        push @comments, $self->format_header_entry( 'sp', $dmarc_sub_policy );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.published-subdomain-policy' )->safe_set_value( $dmarc_sub_policy ) );
+    }
+    if ( $config->{'detect_list_id'} && $self->{'is_list'} ) {
+        push @comments, 'has-list-id=yes';
+    }
+    if ( $dmarc_disposition ) {
+        push @comments, $self->format_header_entry( 'd', $dmarc_disposition );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.applied-disposition' )->safe_set_value( $dmarc_disposition ) );
+    }
+    if ( $dmarc_disposition_evaluated ) {
+        push @comments, $self->format_header_entry( 'd.eval', $dmarc_disposition_evaluated );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.evaluated-disposition' )->safe_set_value( $dmarc_disposition_evaluated ) );
+    }
+    if ( $policy_override ) {
+        push @comments, $self->format_header_entry( 'override', $policy_override );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.override-reason' )->safe_set_value( $policy_override ) );
+    }
+    if ( $arc_aware_result ) {
+        push @comments, $self->format_header_entry( 'arc_aware_result', $arc_aware_result );
+        $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.arc-aware-result' )->safe_set_value( $arc_aware_result ) );
+    }
+
+    if ( @comments ) {
+        $header->add_child( Mail::AuthenticationResults::Header::Comment->new()->safe_set_value( join( ',', @comments ) ) );
+    }
+
+    my $policy_used = ( $is_subdomain && $dmarc_sub_policy ne 'default' ) ? 'sp' : 'p';
+    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'policy.policy-from' )->safe_set_value( $policy_used ) );
+
+    $header->add_child( Mail::AuthenticationResults::Header::SubEntry->new()->set_key( 'header.from' )->safe_set_value( $header_domain ) );
+    $self->_add_dmarc_header( $header );
+
     # Write Metrics
     my $metric_data = {
         'result'           => $dmarc_code,
@@ -662,6 +727,7 @@ sub envfrom_callback {
     return if ( $self->is_trusted_ip_address() );
     return if ( $self->is_authenticated() );
     delete $self->{'from_header'};
+    $self->{'processed'}    = {};
     $self->{'is_list'}      = 0;
     $self->{'skip_report'}  = 0;
     $self->{'failmode'}     = 0;
@@ -822,7 +888,9 @@ sub eom_callback {
 
     if ( @{ $self->{ 'dmarc_ar_headers' } } ) {
         foreach my $dmarc_header ( @{ $self->_get_unique_dmarc_headers() } ) {
-            $self->add_auth_header( $dmarc_header );
+            if ( !( $config->{'hide_none'} && $dmarc_header->value() eq 'none' ) ) {
+                $self->add_auth_header( $dmarc_header );
+            }
         }
     }
     else {
@@ -894,6 +962,49 @@ sub addheader_callback {
     my $handler = shift;
 }
 
+sub dequeue_callback {
+    my ($self) = @_;
+    my $dequeue_list = $self->get_dequeue_list('dmarc_report');
+    foreach my $id ( $dequeue_list->@* ) {
+        my $report = $self->get_dequeue($id);
+        if ( $report ) {
+
+            eval {
+                $self->set_handler_alarm( 5 * 1000000 ); # Allow no longer than 5 seconds for this!
+                if ( $report->can('set_resolver') ) {
+                    my $resolver = $self->get_object('resolver');
+                    $report->set_resolver($resolver);
+                }
+                $report->save_aggregate();
+                $self->dbgout( 'Queued DMARC Report saved for', $report->result()->published()->rua(), LOG_INFO );
+                $self->delete_dequeue($id);
+                $self->reset_alarm();
+            };
+            if ( my $Error = $@ ) {
+                $self->reset_alarm();
+                my $Type = $self->is_exception_type( $Error );
+                if ( $Type ) {
+                    if ( $Type eq 'Timeout' ) {
+                        # We have a timeout, is it global or is it ours?
+                        if ( $self->get_time_remaining() > 0 ) {
+                            # We have time left, but this aggregate save timed out
+                            # Log this and move on!
+                            $self->log_error("DMARC timeout saving reports for $id");
+                        }
+                    }
+                }
+                $self->handle_exception( $Error );
+                $self->log_error("DMARC Report save failed for $id: $Error");
+            }
+
+        }
+        else {
+            $self->log_error("DMARC Report dequeue failed for $id");
+            $self->error_dequeue($id);
+        }
+    }
+}
+
 sub _save_aggregate_reports {
     my ( $self ) = @_;
     return if ! $self->{'report_queue'};
@@ -901,8 +1012,11 @@ sub _save_aggregate_reports {
     eval {
         $self->set_handler_alarm( 2 * 1000000 ); # Allow no longer than 2 seconds for this!
         while ( my $report = shift @{ $self->{'report_queue'} } ) {
-            $report->save_aggregate();
-            $self->dbgout( 'DMARC Report saved for', $report->result()->published()->rua(), LOG_INFO );
+            if ( $report->can('set_resolver') ) {
+                $report->set_resolver(undef);
+            }
+            $self->add_dequeue('dmarc_report',$report);
+            $self->dbgout( 'DMARC Report queued for', $report->result()->published()->rua(), LOG_INFO );
         }
         $self->reset_alarm();
     };
@@ -937,6 +1051,7 @@ sub close_callback {
     delete $self->{'from_header'};
     delete $self->{'from_headers'};
     delete $self->{'report_queue'};
+    delete $self->{'processed'};
     $self->destroy_object('dmarc');
     $self->destroy_object('dmarc_result');
     $self->destroy_object('dmarc_results');
@@ -956,7 +1071,7 @@ Mail::Milter::Authentication::Handler::DMARC - Handler class for DMARC
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 DESCRIPTION
 
@@ -973,11 +1088,25 @@ This handler requires the SPF and DKIM handlers to be installed and active.
             "no_list_reject"        : 0,                   | Do not reject mail detected as mailing list
             "arc_before_list"       : 0,                   | Don't apply above list detection if we have trusted arc
             "no_list_reject_disposition" : "none",         | Disposition to use for mail detected as mailing list (defaults none)
+            "reject_on_multifrom"     : 20,                | Reject mail if we detect more than X DMARC entities to process
+            "quarantine_on_multifrom" : 15,                | Quarantine mail if we detect more than X DMARC entities to process
+            "skip_on_multifrom"       : 10,                | Skip further processing if we detect more than X DMARC entities to process
             "whitelisted"           : [                    | A list of ip addresses or CIDR ranges, or dkim domains
                 "10.20.30.40",                             | for which we do not want to hard reject mail on fail p=reject
                 "dkim:bad.forwarder.com",                  | (valid) DKIM signing domains can also be whitelisted by
                 "20.30.40.0/24"                            | having an entry such as "dkim:domain.com"
             ],
+            "policy_rbl_lookup"     : {                    | Optionally lookup the from domain in a rbl and add a policy entry
+              "foo" : {                                    | the policy to add, this will translate to policy.foo 
+                "rbl" : "foo.rbl.example.com",             | The RBL to use for this lookup
+                "results" : {                              | Mapping of rbl results to policy entries
+                  "127.0.0.1" : "one",                     | A result of IP will give a corresponding policy entry
+                  "127.0.0.2" : "two",
+                  "*" : "star"                             | Fallback to the '*' entry if not found.
+                                                           |   defaults to 'pass' if no entries and no fallback found
+                }
+              }
+            },
             "use_arc"             : 1,                     | Use trusted ARC results if available
             "hide_none"           : 0,                     | Hide auth line if the result is 'none'
             "detect_list_id"      : "1",                   | Detect a list ID and modify the DMARC authentication header
@@ -988,6 +1117,7 @@ This handler requires the SPF and DKIM handlers to be installed and active.
                 "dmarc@example.com"                        | your report from addresses.
             ],
             "no_report"          : "1",                    | If set then we will not attempt to store DMARC reports.
+            "hide_report_to"     : "1",                    | If set, remove envelope_to from DMARC reports
             "config_file"        : "/etc/mail-dmarc.ini"   | Optional path to dmarc config file
         },
 
