@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for Removing headers
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 
 sub default_config {
@@ -33,26 +33,25 @@ sub is_hostname_mine {
 
     return 0 if ! defined $check_hostname;
 
-    my $hostname = $self->get_my_hostname();
-    my ($check_for) = $hostname =~ /^[^\.]+\.(.*)/;
-
     if ( exists( $config->{'hosts_to_remove'} ) ) {
         foreach my $remove_hostname ( @{ $config->{'hosts_to_remove'} } ) {
-            if (
-                substr( lc $check_hostname, ( 0 - length($remove_hostname) ) ) eq
-                lc $remove_hostname )
-            {
+            if ( $check_hostname =~ m/^(.*\.)?\Q${remove_hostname}\E$/i ) {
                 return 1;
             }
         }
     }
 
-    if (
-        substr( lc $check_hostname, ( 0 - length($check_for) ) ) eq
-        lc $check_for )
-    {
+    my $hostname = $self->get_my_hostname();
+    my ($check_for) = $hostname =~ /^[^\.]+\.(.*)/;
+    if ( $check_hostname =~ m/^(.*\.)?\Q${check_for}\E$/i ) {
         return 1;
     }
+
+    my $authserv_id = $self->get_my_authserv_id();
+    if ( fc( $check_hostname ) eq fc( $authserv_id ) ) {
+        return 1;
+    }
+
     return 0;
 }
 
@@ -180,7 +179,7 @@ Mail::Milter::Authentication::Handler::Sanitize - Handler class for Removing hea
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 DESCRIPTION
 
