@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Handler class for Trusted IP addresses
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use base 'Mail::Milter::Authentication::Handler';
 use Net::IP;
 
@@ -28,15 +28,20 @@ sub is_trusted_ip_address {
     my $trusted = 0;
     foreach my $trusted_ip ( @{ $config->{'trusted_ip_list'} } ) {
         my $trusted_obj = Net::IP->new($trusted_ip);
-        my $is_overlap = $ip_obj->overlaps($trusted_obj) || 0;
-        if (
-               $is_overlap == $IP_A_IN_B_OVERLAP
-            || $is_overlap == $IP_B_IN_A_OVERLAP     # Should never happen
-            || $is_overlap == $IP_PARTIAL_OVERLAP    # Should never happen
-            || $is_overlap == $IP_IDENTICAL
-          )
-        {
-            $trusted = 1;
+        if ( !$trusted_obj ) {
+            $self->log_error( 'TrustedIP: Could not parse Trusted IP '.$trusted_ip );
+        }
+        else {
+            my $is_overlap = $ip_obj->overlaps($trusted_obj) || 0;
+            if (
+                   $is_overlap == $IP_A_IN_B_OVERLAP
+                || $is_overlap == $IP_B_IN_A_OVERLAP     # Should never happen
+                || $is_overlap == $IP_PARTIAL_OVERLAP    # Should never happen
+                || $is_overlap == $IP_IDENTICAL
+              )
+            {
+                $trusted = 1;
+            }
         }
     }
     return $trusted;
@@ -79,7 +84,7 @@ Mail::Milter::Authentication::Handler::TrustedIP - Handler class for Trusted IP
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 DESCRIPTION
 
