@@ -4,11 +4,11 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Class for metrics generation
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use Mail::Milter::Authentication::HTDocs;
 use Mail::Milter::Authentication::Metric::Grafana;
 use File::Temp;
-use Prometheus::Tiny::Shared 0.011;
+use Prometheus::Tiny::Shared 0.020;
 use TOML;
 
 
@@ -80,8 +80,7 @@ sub prom {
             $metric_tempfile = $config->{metric_tempfile};
         }
         if ( ! $metric_tempfile ) {
-            my $file_temp = File::Temp->new( UNLINK => 0, SUFFIX => '.sqlite' );
-            $metric_tempfile = $file_temp->filename;
+            $metric_tempfile = $config->{lib_dir}.'/metrics';
         }
         $self->{metric_tempfile} = $metric_tempfile;
     }
@@ -91,10 +90,21 @@ sub prom {
     if ( ! -e $metric_tempfile ) {
         $prom = undef;
     }
+    if ( ! -d $metric_tempfile ) {
+        # If metric_tempfile is a regular file then we need to re-init with a directory
+        # this is likely a restart after upgrade.
+        $prom = undef;
+    }
 
     if ( ! $prom ) {
+        if ( -f $metric_tempfile ) {
+            unlink $metric_tempfile;
+        }
+        if ( ! -d $metric_tempfile ) {
+            mkdir $metric_tempfile, 0700;
+        }
         $self->dbgout( 'Metrics', "Setup new metrics file $metric_tempfile", LOG_DEBUG );
-        $prom = eval{ Prometheus::Tiny::Shared->new(filename => $metric_tempfile, init_file => 1) };
+        $prom = eval{ Prometheus::Tiny::Shared->new(filename => $metric_tempfile.'/authmilter_metrics', init_file => 1) };
         $self->handle_exception($@);
         if ( $prom ) {
             $self->{metric_tempfile} = $metric_tempfile;
@@ -251,7 +261,7 @@ sub _register_metrics {
 }
 
 
-sub master_metric_update {
+sub parent_metric_update {
     my ( $self, $server ) = @_;
     return if ! $self->{enabled};
     return if ! $self->prom;
@@ -365,7 +375,7 @@ sub child_handler {
 
     print $socket qq{</table>
 
-    <h2>Connection/Config Details</h2>
+    <h2>Details</h2>
     <ul>};
     print $socket '<li>Protocol: ' . $config->{protocol} . '</li>';
     my $connections = $config->{connections};
@@ -373,46 +383,66 @@ sub child_handler {
     foreach my $connection ( sort keys %$connections ) {
         print $socket '<li>' . $connection . ': ' . $connections->{ $connection }->{connection} . '</li>'
     }
+    print $socket qq{<li>Effective config (<a href="/config/toml">toml</a>/<a href="/config/json">json</a>)</li>} if !$config->{'metric_basic_http'};
     print $socket qq{
-        <li>Effective config (<a href="/config/toml">toml</a>/<a href="/config/json">json</a>)</li>
     </ul>
 
     <h2>Metrics</h2>
     <ul>
         <li><a href="/metrics">Prometheus metrics endpoint</a></li>
-        <li>Example <a href="/grafana">Grafana dashboard</a> for this setup</li>
+    };
+    print $socket qq{li>Example <a href="/grafana">Grafana dashboard</a> for this setup</li>} if !$config->{'metric_basic_http'};
+    print $socket qq{
     </ul>
 
     <hr />
 
  </div>
 </body>
-};
+    };
         }
         elsif ( $request_uri eq '/config/json' || $request_uri eq '/config' ) {
-            print $socket "HTTP/1.0 200 OK\n";
-            print $socket "Content-Type: text/plain\n";
-            print $socket "\n";
-            my $json = JSON->new();
-            $json->canonical();
-            $json->pretty();
-            print $socket $json->encode( $config );;
+            if ( $config->{'metric_basic_http'} ) {
+                print $socket "HTTP/1.0 403 Denied\n";
+                print $socket "Content-Type: text/plain\n\nDenied by config\n\n";
+            }
+            else {
+                print $socket "HTTP/1.0 200 OK\n";
+                print $socket "Content-Type: text/plain\n";
+                print $socket "\n";
+                my $json = JSON::XS->new();
+                $json->canonical();
+                $json->pretty();
+                print $socket $json->encode( $config );
+            }
         }
         elsif ( $request_uri eq '/config/toml' ) {
-            print $socket "HTTP/1.0 200 OK\n";
-            print $socket "Content-Type: text/plain\n";
-            print $socket "\n";
-            my $toml = TOML::to_toml( $config );
-            $toml =~ s/\n\[/\n\n\[/g;
-            print $socket $toml;
+            if ( $config->{'metric_basic_http'} ) {
+                print $socket "HTTP/1.0 403 Denied\n";
+                print $socket "Content-Type: text/plain\n\nDenied by config\n\n";
+            }
+            else {
+                print $socket "HTTP/1.0 200 OK\n";
+                print $socket "Content-Type: text/plain\n";
+                print $socket "\n";
+                my $toml = TOML::to_toml( $config );
+                $toml =~ s/\n\[/\n\n\[/g;
+                print $socket $toml;
+            }
         }
         elsif ( $request_uri eq '/grafana' ) {
-            print $socket "HTTP/1.0 200 OK\n";
-            print $socket "Content-Type: application/json\n";
-            print $socket "\n";
+            if ( $config->{'metric_basic_http'} ) {
+                print $socket "HTTP/1.0 403 Denied\n";
+                print $socket "Content-Type: text/plain\n\nDenied by config\n\n";
+            }
+            else {
+                print $socket "HTTP/1.0 200 OK\n";
+                print $socket "Content-Type: application/json\n";
+                print $socket "\n";
 
-            my $Grafana = Mail::Milter::Authentication::Metric::Grafana->new();
-            print $socket $Grafana->get_dashboard( $server );
+                my $Grafana = Mail::Milter::Authentication::Metric::Grafana->new();
+                print $socket $Grafana->get_dashboard( $server );
+            }
         }
         else {
             my $htdocs = Mail::Milter::Authentication::HTDocs->new();
@@ -446,7 +476,7 @@ Mail::Milter::Authentication::Metric - Class for metrics generation
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 DESCRIPTION
 
@@ -522,7 +552,7 @@ Send metrics to the parent server process.
 =head2 I<register_metrics( $hash )>
 
 Register a new set of metric types and help texts.
-Called from the master process in the setup phase.
+Called from the parent process in the setup phase.
 
 Expects a hashref of metric description, keyed on metric name.
 
@@ -531,9 +561,9 @@ Expects a hashref of metric description, keyed on metric name.
 Re-register currently registered metrics to ensure backend
 metadata is correct
 
-=head2 I<master_metric_update( $server )>
+=head2 I<parent_metric_update( $server )>
 
-Called in the master process to periodically update some metrics
+Called in the parent process to periodically update some metrics
 
 =head2 I<child_handler( $server )>
 
