@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Milter protocol handling
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use Net::IP;
 
 sub register_metrics {
@@ -86,7 +86,7 @@ sub milter_process_command {
     elsif ( $command eq SMFIC_HEADER ) {
         my $header = $self->milter_split_buffer( $buffer );
         if ( @$header == 1 ) { push @$header , q{}; };
-        my $original = join( ':', @$header );
+        my $original = join( $self->{'headers_include_space'} ? ':': ': ', @$header );
         push @$header, $original;
         $header->[1] =~ s/^\s+//;
         $header->[0] =~ s/^\s+//;
@@ -110,6 +110,7 @@ sub milter_process_command {
             pack('NNN', 2, $actions_reply, $protocol_reply)
         );
         undef $returncode;
+        $self->{'headers_include_space'} = ($protocol_reply & SMFIP_HDR_LEADSPC) != 0;
     }
     elsif ( $command eq SMFIC_RCPT ) {
         my $envrcpt = $self->milter_split_buffer( $buffer );
@@ -286,9 +287,11 @@ sub milter_split_buffer {
 
 sub add_header {
     my ( $self, $header, $value ) = @_;
+    $value =~ s/\015\012/\012/g;
     $self->write_packet( SMFIR_ADDHEADER,
         $header
         . "\0"
+        . ($self->{'headers_include_space'} ? ' ' : '')
         . $value
         . "\0"
     );
@@ -297,10 +300,12 @@ sub add_header {
 sub change_header {
     my ( $self, $header, $index, $value ) = @_;
     $value = '' unless defined($value);
+    $value =~ s/\015\012/\012/g;
     $self->write_packet( SMFIR_CHGHEADER,
         pack('N', $index)
         . $header
         . "\0"
+        . ($self->{'headers_include_space'} ? ' ' : '')
         . $value
         . "\0"
     );
@@ -308,10 +313,12 @@ sub change_header {
 
 sub insert_header {
     my ( $self, $index, $key, $value ) = @_;
+    $value =~ s/\015\012/\012/g;
     $self->write_packet( SMFIR_INSHEADER,
         pack( 'N', $index )
         . $key
         . "\0"
+        . ($self->{'headers_include_space'} ? ' ' : '')
         . $value
         . "\0"
     );
@@ -342,7 +349,7 @@ Mail::Milter::Authentication::Protocol::Milter - Milter protocol handling
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 SYNOPSIS
 
