@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Mail::Milter::Authentication::Pragmas;
 # ABSTRACT: Class used for testing
-our $VERSION = '2.20200319'; # VERSION
+our $VERSION = '3.20210113'; # VERSION
 use Mail::Milter::Authentication;
 use Mail::Milter::Authentication::Client;
 use Mail::Milter::Authentication::Protocol::Milter;
@@ -115,7 +115,7 @@ sub test_metrics {
     subtest $expected => sub {
 
         my $metrics =  get_metrics( 'tmp/authentication_milter_test_metrics.sock' );
-        my $j = JSON->new();
+        my $j = JSON::XS->new();
 
         if ( -e $expected ) {
 
@@ -573,7 +573,7 @@ Mail::Milter::Authentication::Tester - Class used for testing
 
 =head1 VERSION
 
-version 2.20200319
+version 3.20210113
 
 =head1 AUTHOR
 
